@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 1.05.
 package XML::Compile::Schema::XmlReader;
 use vars '$VERSION';
-$VERSION = '0.86';
+$VERSION = '0.87';
 
 
 use strict;
@@ -311,22 +311,8 @@ sub block_handler
     # not be an additional nesting in the output tree.
     if($max ne 'unbounded' && $max==1)
     {   return ($label => $process) if $min==1;
-        my $code =      # $min==0
-        sub { my $tree    = shift or return ();
-              my $starter = $tree->currentChild or return;
-              my @pairs   = try { $process->($tree) };
-              if($@->wasFatal(class => 'misfit'))
-              {   # error is ok, if nothing consumed
-                  trace "misfit $label (optional) ".$@->wasFatal->message;
-                  my $ending = $tree->currentChild;
-                  $@->reportAll if !$ending || $ending!=$starter;
-                  return ();
-              }
-              elsif($@) {$@->reportAll};
-
-              @pairs;
-            };
-         return ($label => bless($code, 'BLOCK'));
+        my $code = sub { $_[0] && $_[0]->currentChild ? $process->($_[0]) : ()};
+        return ($label => bless($code, 'BLOCK'));
     }
 
     if($max ne 'unbounded' && $min>=$max)
@@ -793,9 +779,7 @@ sub attribute_fixed
 
 sub substgroup
 {   my ($path, $args, $type, %do) = @_;
-
     keys %do or return bless sub { () }, 'BLOCK';
-#warn "SUBST $type; ",join '#', @_;
 
     bless
     sub { my $tree  = shift;
